package come.stormborntales.notevault

import android.net.Uri
import android.os.Bundle
import android.webkit.MimeTypeMap
import androidx.activity.ComponentActivity
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.compose.setContent
import androidx.activity.result.PickVisualMediaRequest
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.runtime.*
import androidx.compose.ui.platform.LocalContext
import come.stormborntales.notevault.data.model.NoteEntry
import come.stormborntales.notevault.ui.screens.AddNoteDialog
import come.stormborntales.notevault.ui.screens.MainScreen
import come.stormborntales.notevault.ui.theme.NoteVaultTheme
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.File
import java.io.InputStream

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            NoteVaultTheme {
                val context = LocalContext.current

                // Globale Notenliste
                val notes = remember { mutableStateListOf<NoteEntry>() }

                // Bildauswahl + Dialog-States
                var selectedUris by remember { mutableStateOf<List<Uri>>(emptyList()) }
                var showDialog by remember { mutableStateOf(false) }

                // Launcher innerhalb von Compose
                val imagePickerLauncher = rememberLauncherForActivityResult(
                    contract = ActivityResultContracts.OpenMultipleDocuments(),
                    onResult = { uris ->
                        if (uris.isNotEmpty()) {
                            selectedUris = uris
                            showDialog = true
                        }
                    }
                )

                // UI anzeigen
                MainScreen(
                    notes = notes,
                    onAddNoteClicked = {
                        imagePickerLauncher.launch(
                            arrayOf("image/*")
                        )
                    }
                )

                // Dialog bei Bedarf
                if (showDialog) {
                    val context = LocalContext.current;
                    val scope = rememberCoroutineScope();
                    AddNoteDialog(
                        onDismiss = { showDialog = false },
                        onSave = { title, composer, year, genre, description ->
                            scope.launch(Dispatchers.IO) {
                                val copiedUris = selectedUris.mapNotNull { uri ->
                                    try {
                                        val inputStream: InputStream? = context.contentResolver.openInputStream(uri)
                                        val extension = MimeTypeMap.getSingleton()
                                            .getExtensionFromMimeType(context.contentResolver.getType(uri)) ?: "jpg" // Fallback

                                        val outputFile = File(context.filesDir, "note_${System.currentTimeMillis()}.$extension")
                                        inputStream?.use { input ->
                                            outputFile.outputStream().use { output ->
                                                input.copyTo(output)
                                            }
                                        }
                                        Uri.fromFile(outputFile)
                                    } catch (e: Exception) {
                                        e.printStackTrace()
                                        null
                                    }
                                }

                                if (copiedUris.isNotEmpty()) {
                                    notes.add(
                                        NoteEntry(
                                            title = title,
                                            images = copiedUris,
                                            composer = composer,
                                            year = year,
                                            genre = genre,
                                            description = description
                                        )
                                    )
                                }

                                showDialog = false
                            }


                        }
                    )
                }
            }
        }
    }
}
